# Generated by roxygen2: do not edit by hand

S3method("[",bm_collect)
S3method("[",bm_list)
S3method("[",bm_multi)
S3method("[",bm_sum)
S3method("[",bru_comp_list)
S3method("[",bru_log)
S3method("[",bru_mapper_collect)
S3method("[",bru_mapper_multi)
S3method("[",bru_mapper_sum)
S3method("[",bru_obs_list)
S3method("ibm_names<-",bm_collect)
S3method("ibm_names<-",bm_multi)
S3method("ibm_names<-",bm_sum)
S3method("ibm_names<-",bru_mapper_collect)
S3method("ibm_names<-",bru_mapper_multi)
S3method("ibm_names<-",bru_mapper_sum)
S3method(add_mappers,bru_comp)
S3method(add_mappers,bru_comp_list)
S3method(as.character,bru_log)
S3method(as_bm_list,bm_list)
S3method(as_bm_list,bru_comp_list)
S3method(as_bm_list,list)
S3method(as_bru_comp,bru_comp)
S3method(as_bru_comp_list,bru)
S3method(as_bru_comp_list,bru_comp)
S3method(as_bru_comp_list,bru_comp_list)
S3method(as_bru_comp_list,bru_info)
S3method(as_bru_comp_list,bru_model)
S3method(as_bru_comp_list,formula)
S3method(as_bru_comp_list,list)
S3method(as_bru_info,bru)
S3method(as_bru_info,bru_info)
S3method(as_bru_mapper,bru_comp)
S3method(as_bru_mapper,bru_mapper)
S3method(as_bru_mapper,bru_subcomp)
S3method(as_bru_obs,bru_obs)
S3method(as_bru_obs_list,bru)
S3method(as_bru_obs_list,bru_info)
S3method(as_bru_obs_list,bru_model)
S3method(as_bru_obs_list,bru_obs)
S3method(as_bru_obs_list,bru_obs_list)
S3method(as_bru_obs_list,list)
S3method(bru_comp,character)
S3method(bru_comp_list,bru_comp)
S3method(bru_comp_list,bru_comp_list)
S3method(bru_comp_list,formula)
S3method(bru_comp_list,list)
S3method(bru_compute_linearisation,bru_comp)
S3method(bru_compute_linearisation,bru_model)
S3method(bru_compute_linearisation,bru_obs)
S3method(bru_compute_linearisation,bru_obs_list)
S3method(bru_get_mapper,inla.cgeneric)
S3method(bru_get_mapper,inla.rgeneric)
S3method(bru_get_mapper,inla.spde)
S3method(bru_get_mapper,inla_model_reparam)
S3method(bru_index,bru)
S3method(bru_index,bru_comp)
S3method(bru_index,bru_comp_list)
S3method(bru_index,bru_info)
S3method(bru_index,bru_model)
S3method(bru_index,bru_obs)
S3method(bru_index,bru_obs_list)
S3method(bru_info,bru)
S3method(bru_info,character)
S3method(bru_inla_formula,bru)
S3method(bru_inla_formula,bru_comp_list)
S3method(bru_inla_formula,bru_info)
S3method(bru_input,bm_collect)
S3method(bru_input,bm_multi)
S3method(bru_input,bm_pipe)
S3method(bru_input,bm_repeat)
S3method(bru_input,bm_sum)
S3method(bru_input,bru_comp)
S3method(bru_input,bru_comp_list)
S3method(bru_input,bru_input)
S3method(bru_input,bru_mapper)
S3method(bru_input,bru_model)
S3method(bru_input,bru_obs)
S3method(bru_input,bru_obs_list)
S3method(bru_input_text,bm_collect)
S3method(bru_input_text,bm_multi)
S3method(bru_input_text,bm_pipe)
S3method(bru_input_text,bm_repeat)
S3method(bru_input_text,bm_sum)
S3method(bru_input_text,bru_comp)
S3method(bru_input_text,bru_comp_list)
S3method(bru_input_text,bru_input)
S3method(bru_input_text,bru_mapper)
S3method(bru_is_additive,bru_obs)
S3method(bru_is_additive,bru_obs_list)
S3method(bru_is_additive,bru_pred_expr)
S3method(bru_is_additive,character)
S3method(bru_is_additive,expression)
S3method(bru_is_additive,formula)
S3method(bru_is_linear,bru)
S3method(bru_is_linear,bru_comp)
S3method(bru_is_linear,bru_comp_list)
S3method(bru_is_linear,bru_info)
S3method(bru_is_linear,bru_mapper)
S3method(bru_is_linear,bru_model)
S3method(bru_is_linear,bru_obs)
S3method(bru_is_linear,bru_obs_list)
S3method(bru_is_linear,bru_pred_expr)
S3method(bru_is_rowwise,bru_comp)
S3method(bru_is_rowwise,bru_comp_list)
S3method(bru_is_rowwise,bru_mapper)
S3method(bru_is_rowwise,bru_obs)
S3method(bru_is_rowwise,bru_obs_list)
S3method(bru_is_rowwise,bru_pred_expr)
S3method(bru_log,bru)
S3method(bru_log,bru_log)
S3method(bru_log,character)
S3method(bru_log,iinla)
S3method(bru_make_stack,bru_obs)
S3method(bru_make_stack,bru_obs_list)
S3method(bru_mapper,fm_mesh_1d)
S3method(bru_mapper,fm_mesh_2d)
S3method(bru_names,bru)
S3method(bru_names,inla)
S3method(bru_obs_control_family,bru_obs)
S3method(bru_obs_control_family,bru_obs_list)
S3method(bru_obs_control_gcpo,bru_obs)
S3method(bru_obs_control_gcpo,bru_obs_list)
S3method(bru_obs_inla_family,bru_obs)
S3method(bru_obs_inla_family,bru_obs_list)
S3method(bru_obs_list,bru_obs)
S3method(bru_obs_list,bru_obs_list)
S3method(bru_obs_list,list)
S3method(bru_pred_expr,bru)
S3method(bru_pred_expr,bru_info)
S3method(bru_pred_expr,bru_obs)
S3method(bru_pred_expr,bru_obs_list)
S3method(bru_pred_expr,bru_pred_expr)
S3method(bru_response_size,bru)
S3method(bru_response_size,bru_info)
S3method(bru_response_size,bru_obs)
S3method(bru_response_size,bru_obs_list)
S3method(bru_response_size,default)
S3method(bru_response_size,inla.surv)
S3method(bru_response_size,list)
S3method(bru_set_missing,bru)
S3method(bru_set_missing,bru_info)
S3method(bru_set_missing,bru_obs)
S3method(bru_set_missing,bru_obs_list)
S3method(bru_set_missing,data.frame)
S3method(bru_set_missing,default)
S3method(bru_set_missing,inla.surv)
S3method(bru_timings,bru)
S3method(bru_used,"NULL")
S3method(bru_used,bru)
S3method(bru_used,bru_info)
S3method(bru_used,bru_obs)
S3method(bru_used,bru_pred_expr)
S3method(bru_used,bru_used)
S3method(bru_used,default)
S3method(bru_used,list)
S3method(bru_used_update,bru_obs)
S3method(bru_used_update,bru_obs_list)
S3method(bru_used_update,bru_pred_expr)
S3method(bru_used_update,bru_used)
S3method(bru_used_vars,character)
S3method(bru_used_vars,expression)
S3method(bru_used_vars,formula)
S3method(bru_used_vars,quosure)
S3method(c,bm_list)
S3method(c,bru_comp)
S3method(c,bru_comp_list)
S3method(c,bru_log)
S3method(c,bru_mapper)
S3method(c,bru_obs)
S3method(c,bru_obs_list)
S3method(eval_spatial,SpatRaster)
S3method(eval_spatial,SpatialGridDataFrame)
S3method(eval_spatial,SpatialPixelsDataFrame)
S3method(eval_spatial,SpatialPolygonsDataFrame)
S3method(eval_spatial,sf)
S3method(eval_spatial,stars)
S3method(evaluate_effect_single_state,bm_list)
S3method(evaluate_effect_single_state,bru_comp_list)
S3method(evaluate_effect_single_state,bru_mapper)
S3method(format,bm_collect)
S3method(format,bm_list)
S3method(format,bm_multi)
S3method(format,bm_pipe)
S3method(format,bm_reparam)
S3method(format,bm_repeat)
S3method(format,bm_sum)
S3method(format,bru_input)
S3method(format,bru_log)
S3method(format,bru_mapper)
S3method(format,bru_pred_expr)
S3method(format,bru_subcomp)
S3method(format,bru_used)
S3method(generate,bru)
S3method(gg,RasterLayer)
S3method(gg,SpatRaster)
S3method(gg,SpatialGridDataFrame)
S3method(gg,SpatialLines)
S3method(gg,SpatialPixels)
S3method(gg,SpatialPixelsDataFrame)
S3method(gg,SpatialPoints)
S3method(gg,SpatialPolygons)
S3method(gg,bru_prediction)
S3method(gg,data.frame)
S3method(gg,fm_mesh_1d)
S3method(gg,fm_mesh_2d)
S3method(gg,matrix)
S3method(gg,prediction)
S3method(gg,sf)
S3method(glplot,SpatialLines)
S3method(glplot,SpatialPoints)
S3method(glplot,fm_mesh_2d)
S3method(ibm_eval,bm_aggregate)
S3method(ibm_eval,bm_collect)
S3method(ibm_eval,bm_const)
S3method(ibm_eval,bm_logitaverage)
S3method(ibm_eval,bm_logsumexp)
S3method(ibm_eval,bm_marginal)
S3method(ibm_eval,bm_multi)
S3method(ibm_eval,bm_pipe)
S3method(ibm_eval,bm_reparam)
S3method(ibm_eval,bm_repeat)
S3method(ibm_eval,bm_scale)
S3method(ibm_eval,bm_shift)
S3method(ibm_eval,bm_sum)
S3method(ibm_eval,bm_taylor)
S3method(ibm_eval,default)
S3method(ibm_eval2,bm_pipe)
S3method(ibm_eval2,default)
S3method(ibm_inla_subset,default)
S3method(ibm_invalid_output,bm_collect)
S3method(ibm_invalid_output,bm_index)
S3method(ibm_invalid_output,bm_multi)
S3method(ibm_invalid_output,bm_repeat)
S3method(ibm_invalid_output,bm_sum)
S3method(ibm_invalid_output,default)
S3method(ibm_is_linear,bm_collect)
S3method(ibm_is_linear,bm_multi)
S3method(ibm_is_linear,bm_sum)
S3method(ibm_is_linear,default)
S3method(ibm_is_rowwise,default)
S3method(ibm_jacobian,bm_aggregate)
S3method(ibm_jacobian,bm_collect)
S3method(ibm_jacobian,bm_const)
S3method(ibm_jacobian,bm_factor)
S3method(ibm_jacobian,bm_fm_mesh_1d)
S3method(ibm_jacobian,bm_fm_mesh_2d)
S3method(ibm_jacobian,bm_fmesher)
S3method(ibm_jacobian,bm_harmonics)
S3method(ibm_jacobian,bm_index)
S3method(ibm_jacobian,bm_inla_mesh_1d)
S3method(ibm_jacobian,bm_inla_mesh_2d)
S3method(ibm_jacobian,bm_linear)
S3method(ibm_jacobian,bm_logitaverage)
S3method(ibm_jacobian,bm_logsumexp)
S3method(ibm_jacobian,bm_marginal)
S3method(ibm_jacobian,bm_matrix)
S3method(ibm_jacobian,bm_multi)
S3method(ibm_jacobian,bm_pipe)
S3method(ibm_jacobian,bm_reparam)
S3method(ibm_jacobian,bm_repeat)
S3method(ibm_jacobian,bm_scale)
S3method(ibm_jacobian,bm_shift)
S3method(ibm_jacobian,bm_sum)
S3method(ibm_jacobian,bm_taylor)
S3method(ibm_jacobian,default)
S3method(ibm_linear,bm_collect)
S3method(ibm_linear,bm_list)
S3method(ibm_linear,bm_multi)
S3method(ibm_linear,bm_repeat)
S3method(ibm_linear,bm_sum)
S3method(ibm_linear,bru_comp)
S3method(ibm_linear,bru_comp_list)
S3method(ibm_linear,bru_model)
S3method(ibm_linear,default)
S3method(ibm_n,bm_aggregate)
S3method(ibm_n,bm_collect)
S3method(ibm_n,bm_const)
S3method(ibm_n,bm_factor)
S3method(ibm_n,bm_fm_mesh_1d)
S3method(ibm_n,bm_fm_mesh_2d)
S3method(ibm_n,bm_fmesher)
S3method(ibm_n,bm_harmonics)
S3method(ibm_n,bm_inla_mesh_1d)
S3method(ibm_n,bm_inla_mesh_2d)
S3method(ibm_n,bm_linear)
S3method(ibm_n,bm_marginal)
S3method(ibm_n,bm_matrix)
S3method(ibm_n,bm_multi)
S3method(ibm_n,bm_pipe)
S3method(ibm_n,bm_reparam)
S3method(ibm_n,bm_repeat)
S3method(ibm_n,bm_scale)
S3method(ibm_n,bm_shift)
S3method(ibm_n,bm_sum)
S3method(ibm_n,bm_taylor)
S3method(ibm_n,default)
S3method(ibm_n_output,bm_aggregate)
S3method(ibm_n_output,bm_collect)
S3method(ibm_n_output,bm_marginal)
S3method(ibm_n_output,bm_multi)
S3method(ibm_n_output,bm_pipe)
S3method(ibm_n_output,bm_reparam)
S3method(ibm_n_output,bm_repeat)
S3method(ibm_n_output,bm_scale)
S3method(ibm_n_output,bm_shift)
S3method(ibm_n_output,bm_sum)
S3method(ibm_n_output,bm_taylor)
S3method(ibm_n_output,default)
S3method(ibm_names,bm_collect)
S3method(ibm_names,bm_multi)
S3method(ibm_names,bm_sum)
S3method(ibm_names,default)
S3method(ibm_simplify,bm_list)
S3method(ibm_simplify,bm_pipe)
S3method(ibm_simplify,bru_comp)
S3method(ibm_simplify,bru_comp_list)
S3method(ibm_simplify,bru_model)
S3method(ibm_simplify,default)
S3method(ibm_values,bm_aggregate)
S3method(ibm_values,bm_collect)
S3method(ibm_values,bm_const)
S3method(ibm_values,bm_factor)
S3method(ibm_values,bm_fm_mesh_1d)
S3method(ibm_values,bm_fm_mesh_2d)
S3method(ibm_values,bm_fmesher)
S3method(ibm_values,bm_inla_mesh_1d)
S3method(ibm_values,bm_inla_mesh_2d)
S3method(ibm_values,bm_linear)
S3method(ibm_values,bm_marginal)
S3method(ibm_values,bm_matrix)
S3method(ibm_values,bm_multi)
S3method(ibm_values,bm_pipe)
S3method(ibm_values,bm_reparam)
S3method(ibm_values,bm_repeat)
S3method(ibm_values,bm_scale)
S3method(ibm_values,bm_shift)
S3method(ibm_values,bm_sum)
S3method(ibm_values,bm_taylor)
S3method(ibm_values,default)
S3method(inla_subset_eval,bru_comp_list)
S3method(length,bru_log)
S3method(new_bru_used,"NULL")
S3method(new_bru_used,character)
S3method(new_bru_used,expression)
S3method(new_bru_used,formula)
S3method(new_bru_used,quosure)
S3method(plot,bru)
S3method(plot,bru_prediction)
S3method(plot,prediction)
S3method(predict,bru)
S3method(print,bm_list)
S3method(print,bru)
S3method(print,bru_comp)
S3method(print,bru_comp_list)
S3method(print,bru_info)
S3method(print,bru_input)
S3method(print,bru_log)
S3method(print,bru_mapper)
S3method(print,bru_model)
S3method(print,bru_obs)
S3method(print,bru_obs_list)
S3method(print,bru_pred_expr)
S3method(print,bru_subcomp)
S3method(print,bru_used)
S3method(print,summary_bru)
S3method(print,summary_bru_comp_list)
S3method(print,summary_bru_info)
S3method(print,summary_bru_input)
S3method(print,summary_bru_mapper)
S3method(print,summary_bru_model)
S3method(print,summary_bru_obs)
S3method(print,summary_bru_obs_list)
S3method(print,summary_bru_options)
S3method(print,summary_bru_subcomp)
S3method(print,summary_component)
S3method(summary,bru)
S3method(summary,bru_comp)
S3method(summary,bru_comp_list)
S3method(summary,bru_info)
S3method(summary,bru_input)
S3method(summary,bru_mapper)
S3method(summary,bru_model)
S3method(summary,bru_obs)
S3method(summary,bru_obs_list)
S3method(summary,bru_options)
S3method(summary,bru_subcomp)
S3method(summary,bru_used)
export("bru_comp_env<-")
export("bru_comp_env_extra<-")
export("bru_set_missing<-")
export("ibm_names<-")
export(add_mappers)
export(as.bru_options)
export(as_bm_list)
export(as_bru_comp)
export(as_bru_comp_list)
export(as_bru_info)
export(as_bru_mapper)
export(as_bru_obs)
export(as_bru_obs_list)
export(bincount)
export(bm_aggregate)
export(bm_autodetect)
export(bm_collect)
export(bm_const)
export(bm_factor)
export(bm_fmesher)
export(bm_harmonics)
export(bm_index)
export(bm_linear)
export(bm_logitaverage)
export(bm_logsumexp)
export(bm_marginal)
export(bm_matrix)
export(bm_mesh_B)
export(bm_multi)
export(bm_pipe)
export(bm_reparam)
export(bm_repeat)
export(bm_repeat_indexing)
export(bm_repeat_indexing_matrix)
export(bm_scale)
export(bm_shift)
export(bm_sum)
export(bm_taylor)
export(bru)
export(bru_call_options)
export(bru_comp)
export(bru_comp_env)
export(bru_comp_env_extra)
export(bru_comp_list)
export(bru_compute_linearisation)
export(bru_convergence_plot)
export(bru_eval_in_data_context)
export(bru_fill_missing)
export(bru_forward_transformation)
export(bru_get_mapper)
export(bru_get_mapper_safely)
export(bru_index)
export(bru_info)
export(bru_inla.stack.mexpand)
export(bru_inla.stack.mjoin)
export(bru_inla_formula)
export(bru_input)
export(bru_input_text)
export(bru_inverse_transformation)
export(bru_is_additive)
export(bru_is_linear)
export(bru_is_rowwise)
export(bru_like_list)
export(bru_log)
export(bru_log_abort)
export(bru_log_bookmark)
export(bru_log_bookmarks)
export(bru_log_index)
export(bru_log_message)
export(bru_log_new)
export(bru_log_offset)
export(bru_log_reset)
export(bru_log_warn)
export(bru_make_stack)
export(bru_mapper)
export(bru_mapper_aggregate)
export(bru_mapper_collect)
export(bru_mapper_const)
export(bru_mapper_define)
export(bru_mapper_factor)
export(bru_mapper_fmesher)
export(bru_mapper_harmonics)
export(bru_mapper_index)
export(bru_mapper_linear)
export(bru_mapper_logsumexp)
export(bru_mapper_marginal)
export(bru_mapper_matrix)
export(bru_mapper_mesh_B)
export(bru_mapper_multi)
export(bru_mapper_pipe)
export(bru_mapper_repeat)
export(bru_mapper_scale)
export(bru_mapper_shift)
export(bru_mapper_sum)
export(bru_mapper_taylor)
export(bru_model)
export(bru_names)
export(bru_obs)
export(bru_obs_control_family)
export(bru_obs_control_gcpo)
export(bru_obs_inla_family)
export(bru_obs_list)
export(bru_options)
export(bru_options_check)
export(bru_options_default)
export(bru_options_get)
export(bru_options_reset)
export(bru_options_set)
export(bru_options_set_local)
export(bru_pred_expr)
export(bru_rerun)
export(bru_response_size)
export(bru_safe_inla)
export(bru_safe_sp)
export(bru_safe_terra)
export(bru_set_missing)
export(bru_standardise_names)
export(bru_summarise)
export(bru_timings)
export(bru_timings_plot)
export(bru_used)
export(bru_used_update)
export(bru_used_vars)
export(cv_hex)
export(cv_partition)
export(deltaIC)
export(delta_IC)
export(devel.cvmeasure)
export(eval_spatial)
export(evaluate_effect_single_state)
export(evaluate_model)
export(evaluate_state)
export(fm_int)
export(fm_pixels)
export(generate)
export(gg)
export(globe)
export(glplot)
export(gmap)
export(gorillas_sf_gcov)
export(gorillas_sp)
export(ibm_eval)
export(ibm_eval2)
export(ibm_inla_subset)
export(ibm_input_available)
export(ibm_input_get)
export(ibm_input_new)
export(ibm_input_set)
export(ibm_invalid_output)
export(ibm_is_linear)
export(ibm_is_rowwise)
export(ibm_jacobian)
export(ibm_linear)
export(ibm_n)
export(ibm_n_output)
export(ibm_names)
export(ibm_simplify)
export(ibm_values)
export(iinla)
export(index_eval)
export(inla.spde2.pcmatern_B)
export(inla_subset_eval)
export(input_eval)
export(lgcp)
export(lgcp_IC)
export(like)
export(like_list)
export(local_basic_fixed_effect_testdata)
export(local_basic_intercept_testdata)
export(local_bru_options_set)
export(local_bru_safe_inla)
export(local_bru_testthat_assign)
export(local_bru_testthat_setup)
export(local_bru_testthat_tolerances)
export(make_hierarchical_mesh_basis)
export(materncov.bands)
export(mexdolphin_sp)
export(multiplot)
export(new_bru_input)
export(new_bru_pred_expr)
export(new_bru_used)
export(plotmarginal.inla)
export(plotsample)
export(point2count)
export(row_kron)
export(sample.lgcp)
export(sline)
export(spatial.to.ppp)
export(spde.posterior)
export(spoly)
import(fmesher)
import(methods)
import(stats)
importFrom(Matrix,diag)
importFrom(Rcpp,sourceCpp)
importFrom(fmesher,fm_int)
importFrom(fmesher,fm_pixels)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(glue,glue_data)
importFrom(graphics,hist)
importFrom(lifecycle,deprecated)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(utils,modifyList)
useDynLib(inlabru, .registration = TRUE)
