#' Deprecated functions in inlabru
#'
#' Some of these functions may still attempt to do their job, but will be
#' removed in a future version.
#'
#' @param \dots Usually passed on to other methods
#'
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#'
#' @name inlabru-deprecated
NULL
