# 2D LGCP fitting

    WAoAAAACAAQFAgACAwAAAAAOAAAABkAMzarvzFUeQAC35LPWQOU/+W5HC7kV0UAR0MD1iAO3
    QAzLB4wnOLRAD22P8nzFDg==

---

    WAoAAAACAAQFAgACAwAAAAAOAAAABj/jj+60Uw0mP74yXKqnqtI/y/rFb3uoNz+mqJJQha2e
    P8red6Fef+Y/z11PkiF1aA==

