# 2D LGCP fitting (sf)

    WAoAAAACAAQFAgACAwAAAAAOAAAABb/jCNZYO5mDQAtPqPCJIW8/ytHux+g7Kb/JKJJKBb1R
    v++BgRD8VYA=

---

    WAoAAAACAAQFAgACAwAAAAAOAAAABT/8SYedodjqP+OA5s8ws6U/6LBi1gMiLD/pfrKl1tWw
    P/R7efxhv4A=

---

    WAoAAAACAAQFAgACAwAAAAAOAAAABL/W/TEYi4bWv4nkmJPX5Ia/ypmHVl02HL/fyfyioJmg
    

---

    WAoAAAACAAQFAQACAwAAAAAOAAAABD/lvtZhqfQWP+tl+nXWUrw/9LjvMOUXAEABA9yVDC5v
    

# 2D LGCP fitting (sp)

    WAoAAAACAAQFAgACAwAAAAAOAAAABb/jCNZYO5mDQAtPqPCJIW8/ytHux+g7Kb/JKJJKBb1R
    v++BgRD8VYA=

---

    WAoAAAACAAQFAgACAwAAAAAOAAAABT/8SYedodjqP+OA5s8ws6U/6LBi1gMiLD/pfrKl1tWw
    P/R7efxhv4A=

---

    WAoAAAACAAQFAgACAwAAAAAOAAAABL/W/TEYi4bWv4nkmJPX5Ia/ypmHVl02HL/fyfyioJmg
    

---

    WAoAAAACAAQFAQACAwAAAAAOAAAABD/lvtZhqfQWP+tl+nXWUrw/9LjvMOUXAEABA9yVDC5v
    

