% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru.inference.R
\name{summary.bru_obs}
\alias{summary.bru_obs}
\alias{summary.bru_obs_list}
\alias{print.summary_bru_obs}
\alias{print.summary_bru_obs_list}
\alias{print.bru_obs}
\alias{print.bru_obs_list}
\title{Summary and print methods for observation models}
\usage{
\method{summary}{bru_obs}(object, verbose = TRUE, ...)

\method{summary}{bru_obs_list}(object, verbose = TRUE, ...)

\method{print}{summary_bru_obs}(x, ...)

\method{print}{summary_bru_obs_list}(x, ...)

\method{print}{bru_obs}(x, ...)

\method{print}{bru_obs_list}(x, ...)
}
\arguments{
\item{object}{Object to operate on}

\item{verbose}{logical; If \code{TRUE}, include more details of the
component definitions. If \code{FALSE}, only show basic component
definition information. Default: \code{TRUE}}

\item{\dots}{Arguments passed on to other \code{summary} methods}

\item{x}{Object to be printed}
}
\description{
Summary and print methods for observation models
}
\examples{
obs <- bru_obs(y ~ ., data = data.frame(y = rnorm(10)))
summary(obs)
print(obs)

}
\seealso{
\code{\link[=bru_obs]{bru_obs()}}
}
