% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{bru_mapper_generics}
\alias{bru_mapper_generics}
\title{Generic methods for bru_mapper objects}
\description{
A \code{bru_mapper} sub-class implementation must provide an
\code{\link[=ibm_jacobian]{ibm_jacobian()}} method. If the model size 'n' and definition
values 'values' are stored in the object itself, default methods \code{\link[=ibm_n]{ibm_n()}}
and \code{\link[=ibm_values]{ibm_values()}} are
available. Otherwise the
\code{\link[=ibm_n]{ibm_n()}} and \code{\link[=ibm_values]{ibm_values()}} methods also need to be provided.
}
\seealso{
\link{bru_mapper} for constructor methods, and
\link{bru_get_mapper} for hooks to extract mappers from latent model object
class objects.

Other mapper methods: 
\code{\link{ibm_eval}()},
\code{\link{ibm_eval2}()},
\code{\link{ibm_inla_subset}()},
\code{\link{ibm_invalid_output}()},
\code{\link{ibm_is_linear}()},
\code{\link{ibm_is_rowwise}()},
\code{\link{ibm_jacobian}()},
\code{\link{ibm_linear}()},
\code{\link{ibm_n}()},
\code{\link{ibm_n_output}()},
\code{\link{ibm_names}()},
\code{\link{ibm_simplify}()},
\code{\link{ibm_values}()}
}
\concept{mapper methods}
