% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru_input.R
\name{bru_input_text}
\alias{bru_input_text}
\alias{bru_input_text.bru_input}
\alias{bru_input_text.bru_comp}
\alias{bru_input_text.bru_comp_list}
\alias{bru_input_text.bru_mapper}
\alias{bru_input_text.bm_pipe}
\alias{bru_input_text.bm_multi}
\alias{bru_input_text.bm_collect}
\alias{bru_input_text.bm_repeat}
\alias{bru_input_text.bm_sum}
\title{Extract input definitions as text}
\usage{
bru_input_text(...)

\method{bru_input_text}{bru_input}(x, .envir = parent.frame(), ...)

\method{bru_input_text}{bru_comp}(x, ..., label = x$label)

\method{bru_input_text}{bru_comp_list}(x, ...)

\method{bru_input_text}{bru_mapper}(x, ..., label = "<unknown>")

\method{bru_input_text}{bm_pipe}(x, ..., label = "<unknown>")

\method{bru_input_text}{bm_multi}(x, ..., label = "<unknown>")

\method{bru_input_text}{bm_collect}(x, ..., label = "<unknown>")

\method{bru_input_text}{bm_repeat}(x, ..., label = "<unknown>")

\method{bru_input_text}{bm_sum}(x, ..., label = "<unknown>")
}
\arguments{
\item{...}{Passed on to sub-methods.}

\item{x}{A \code{bru_input} object, or other object for recursive evaluation.}

\item{.envir}{environment in which to evaluate the input expression. Default
is \code{parent.frame()}}

\item{label}{character; optional label used to identify the object in
informational messages.}
}
\value{
\itemize{
\item \code{bru_input(bru_comp)}: A character vector of mapper input values.
}

\itemize{
\item \code{bru_input_text(bru_comp_list)}: A list of mapper input definition
text strings, with one entry for each component.
}
}
\description{
Extract input definitions from the \link{bru_input} information stored in bru
model components.  Primarily intended for diagnostic use, as the output does
not currently contain all information, such as the layer and selector
information for spatial covariates.
}
\section{Methods (by class)}{
\itemize{
\item \code{bru_input_text(bru_input)}: Extract the input definitions from a \code{bru_input}
object, as a named character vector

\item \code{bru_input_text(bru_comp)}: Extract the input definition as a named character
vector.

\item \code{bru_input_text(bru_mapper)}: Extract the input definitions associated with a
\code{bru_mapper}.

}}
\examples{
(inp <- new_bru_input(x, "LABEL"))
bru_input_text(inp)
if (bru_safe_inla()) {
  bru_input_text(bru_comp("x", cos(y)))
}

}
\seealso{
\code{\link[=bru_input]{bru_input()}}

\link{ibm_input}
}
\keyword{internal}
