% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{expand_labels}
\alias{expand_labels}
\title{Expand labels}
\usage{
expand_labels(labels, expand, suffix)
}
\arguments{
\item{labels}{character vector; original labels}

\item{expand}{character vector; subset of labels to expand}

\item{suffix}{character; the suffix to add to the labels selected by \code{expand}}
}
\value{
a vector of labels with suffix appended to the selected labels
}
\description{
Expand labels
}
\keyword{internal}
