% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru_index.R
\name{inla_subset_eval}
\alias{inla_subset_eval}
\alias{inla_subset_eval.bru_comp_list}
\title{Obtain inla index subset information}
\usage{
inla_subset_eval(...)

\method{inla_subset_eval}{bru_comp_list}(components, ...)
}
\arguments{
\item{\dots}{Passed on to submethods.}

\item{components}{A component list.}
}
\description{
Subsets for \code{INLA::f()} compatible indexing
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}
\keyword{internal}
