% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/effect.R
\name{bru_formula_to_bru_obs_code}
\alias{bru_formula_to_bru_obs_code}
\title{Convert components to R code}
\usage{
bru_formula_to_bru_obs_code(components, add = "")
}
\arguments{
\item{components}{A \link{formula} describing latent model components.}
}
\description{
Convert components to R code
}
\author{
Fabian E. Bachl \email{bachlfab@gmail.com}
}
\keyword{internal}
