% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{as_bru_mapper}
\alias{as_bru_mapper}
\alias{as_bru_mapper.bru_mapper}
\alias{as_bru_mapper.bru_comp}
\alias{as_bru_mapper.bru_subcomp}
\title{Methods for mapper extraction}
\usage{
as_bru_mapper(x)

\method{as_bru_mapper}{bru_mapper}(x)

\method{as_bru_mapper}{bru_comp}(x)

\method{as_bru_mapper}{bru_subcomp}(x)
}
\arguments{
\item{x}{Object to convert/extract}
}
\value{
A \code{bru_mapper} object
}
\description{
Extract a mapper from another object
}
\examples{
# Extract a mapper from a `bru_subcomp` object
as_bru_mapper(bru_comp("x", x, mapper = bm_index(4))$main)

}
