% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru.inference.R
\name{parse_inclusion}
\alias{parse_inclusion}
\title{Parse inclusion of component labels in a predictor expression}
\usage{
parse_inclusion(thenames, include = NULL, exclude = NULL)
}
\arguments{
\item{thenames}{Set of labels to restrict}

\item{include}{Character vector of component labels that are needed by the
predictor expression; Default: NULL (include all components that are not
explicitly excluded)}

\item{exclude}{Character vector of component labels that are not used by the
predictor expression. The exclusion list is applied to the list
as determined by the \code{include} parameter; Default: NULL (do not remove
any components from the inclusion list)}
}
\description{
Parse inclusion of component labels in a predictor expression
}
\keyword{internal}
