% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inla.R
\name{bru_standardise_names}
\alias{bru_standardise_names}
\title{Standardise inla hyperparameter names}
\usage{
bru_standardise_names(x)
}
\arguments{
\item{x}{character vector; names to be standardised}
}
\value{
A character vector with standardised names
}
\description{
The inla hyperparameter output uses parameter names that can include
whitespace and special characters. This function replaces those characters
with underscores.
}
\examples{
bru_standardise_names("Precision for the Gaussian observations")
}
\seealso{
\code{\link[=bru_names]{bru_names()}}
}
\keyword{internal}
