% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{bm_fm_mesh_old}
\alias{bm_fm_mesh_old}
\alias{ibm_n.bm_fm_mesh_2d}
\alias{ibm_values.bm_fm_mesh_2d}
\alias{ibm_jacobian.bm_fm_mesh_2d}
\alias{ibm_n.bm_inla_mesh_2d}
\alias{ibm_values.bm_inla_mesh_2d}
\alias{ibm_jacobian.bm_inla_mesh_2d}
\alias{ibm_n.bm_inla_mesh_1d}
\alias{ibm_values.bm_inla_mesh_1d}
\alias{ibm_jacobian.bm_inla_mesh_1d}
\title{Deprecated methods}
\usage{
\method{ibm_n}{bm_fm_mesh_2d}(mapper, ...)

\method{ibm_values}{bm_fm_mesh_2d}(mapper, ...)

\method{ibm_jacobian}{bm_fm_mesh_2d}(mapper, input, ...)

\method{ibm_n}{bm_inla_mesh_2d}(mapper, ...)

\method{ibm_values}{bm_inla_mesh_2d}(mapper, ...)

\method{ibm_jacobian}{bm_inla_mesh_2d}(mapper, input, ...)

\method{ibm_n}{bm_inla_mesh_1d}(mapper, ...)

\method{ibm_values}{bm_inla_mesh_1d}(mapper, ...)

\method{ibm_jacobian}{bm_inla_mesh_1d}(mapper, input, ...)
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
Deprecated methods only used for old stored
\code{bru_mapper_fm_mesh_1d} and \code{bru_mapper_fm_mesh_2d} objects.
}
\keyword{internal}
