% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru_input.R
\name{format.bru_input}
\alias{format.bru_input}
\alias{summary.bru_input}
\alias{print.bru_input}
\title{Summarise component inputs}
\usage{
\method{format}{bru_input}(x, verbose = TRUE, ..., label.override = NULL, type = NULL)

\method{summary}{bru_input}(object, verbose = TRUE, ..., label.override = NULL)

\method{print}{bru_input}(x, verbose = TRUE, ..., label.override = NULL)
}
\arguments{
\item{x}{Object to be printed}

\item{verbose}{logical; If \code{TRUE}, includes more details of the
component definitions. When \code{FALSE}, only show basic component
definition information.  Default \code{TRUE}.}

\item{\dots}{Passed on to other summary methods.}

\item{label.override}{character; If not \code{NULL}, use this label instead of
the object's label.}

\item{type}{character; if non-NULL, added to the output'; \code{label = type(input)}.}

\item{object}{Object to be summarised.}
}
\description{
Summarise component inputs
}
\seealso{
\code{\link[=bru_input]{bru_input()}}, \code{\link[=bru_comp]{bru_comp()}}
}
\author{
Fabian E. Bachl \email{bachlfab@gmail.com}

Finn Lindgren \email{finn.lindgren@gmail.com}
}
