% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru.inference.R
\name{bru_inla_formula}
\alias{bru_inla_formula}
\alias{bru_inla_formula.bru}
\alias{bru_inla_formula.bru_info}
\alias{bru_inla_formula.bru_comp_list}
\title{Extract INLA formula}
\usage{
bru_inla_formula(x, ...)

\method{bru_inla_formula}{bru}(x, ...)

\method{bru_inla_formula}{bru_info}(x, ...)

\method{bru_inla_formula}{bru_comp_list}(x, ...)
}
\arguments{
\item{x}{An object containing information about an INLA formula}

\item{\dots}{Additional arguments passed on to submethods}
}
\description{
Extracts the INLA formula used in a \code{bru} model
}
\keyword{internal}
