% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nlinla.R
\name{bru_compute_linearisation}
\alias{bru_compute_linearisation}
\alias{bru_compute_linearisation.bru_comp}
\alias{bru_compute_linearisation.bru_obs}
\alias{bru_compute_linearisation.bru_obs_list}
\alias{bru_compute_linearisation.bru_model}
\title{Compute inlabru model linearisation information}
\usage{
bru_compute_linearisation(...)

\method{bru_compute_linearisation}{bru_comp}(
  cmp,
  model,
  lhood_expr,
  data,
  data_extra,
  input,
  state,
  comp_simple,
  effects,
  pred0,
  used,
  is_rowwise,
  eps,
  n_pred = NULL,
  ...
)

\method{bru_compute_linearisation}{bru_obs}(
  lhood,
  model,
  data,
  input,
  state,
  comp_simple,
  eps,
  ...
)

\method{bru_compute_linearisation}{bru_obs_list}(
  lhoods,
  model,
  input,
  state,
  comp_simple,
  eps = 1e-05,
  ...
)

\method{bru_compute_linearisation}{bru_model}(model, lhoods, input, state, comp_simple, ...)
}
\arguments{
\item{\dots}{Parameters passed on to other methods}

\item{cmp}{A \link{bru_comp} object}

\item{model}{A \code{bru_model} object}

\item{lhood_expr}{A predictor expression}

\item{data}{Input data}

\item{data_extra}{Additional data for the predictor}

\item{input}{Precomputed component inputs from \code{bru_input()}}

\item{state}{The state information, as a list of named vectors}

\item{comp_simple}{Component evaluation information
\itemize{
\item For \code{bru_comp}: A \link{bm_taylor} object
\item For \code{bru_obs}: A \link{bm_list} object
for the components in the likelihood
\item For \code{bru_obs_list}: A list of \link{bm_list} objects
}}

\item{effects}{\itemize{
\item For \code{bru_comp}:
Precomputed effect list for all components involved in the likelihood
expression
}}

\item{pred0}{Precomputed predictor for the given state}

\item{used}{A \code{\link[=bru_used]{bru_used()}} object for the predictor expression}

\item{is_rowwise}{logical; If \code{FALSE}, the predictor expression may
involve several rows of the input data to influence the same row.}

\item{eps}{The finite difference step size}

\item{n_pred}{The length of the predictor expression. If not \code{NULL}, scalar
predictor evaluations are expanded to vectors of length \code{n_pred}.}

\item{lhood}{A \code{bru_obs} object}

\item{lhoods}{A \code{bru_obs_list} object}
}
\description{
Compute inlabru model linearisation information
}
\keyword{internal}
