% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{bm_shift}
\alias{bm_shift}
\alias{bru_mapper_shift}
\title{Mapper for element-wise shifting}
\usage{
bm_shift(mapper = NULL)

bru_mapper_shift(...)
}
\arguments{
\item{mapper}{If non-NULL, a \code{bru_mapper} to be shifted.}

\item{\dots}{Arguments passed on to \code{\link[=bm_shift]{bm_shift()}}}
}
\description{
Create a standalone
shift mapper that can be used as part of a \code{bm_pipe}.
If \code{mapper} is non-null, the \code{bm_shift()} constructor
returns
\code{bm_pipe(list(mapper = mapper, shift = bm_shift()))}
}
\examples{
m <- bm_shift()
ibm_eval2(m, c(1, 2, 1, 2), 1:4)

}
\seealso{
\link{bru_mapper}, \link{bru_mapper_generics}

Other mappers: 
\code{\link{bm_aggregate}()},
\code{\link{bm_collect}()},
\code{\link{bm_const}()},
\code{\link{bm_factor}()},
\code{\link{bm_fm_mesh_1d}},
\code{\link{bm_fmesher}()},
\code{\link{bm_harmonics}()},
\code{\link{bm_index}()},
\code{\link{bm_linear}()},
\code{\link{bm_logitaverage}()},
\code{\link{bm_logsumexp}()},
\code{\link{bm_marginal}()},
\code{\link{bm_matrix}()},
\code{\link{bm_multi}()},
\code{\link{bm_pipe}()},
\code{\link{bm_reparam}()},
\code{\link{bm_repeat}()},
\code{\link{bm_scale}()},
\code{\link{bm_sum}()},
\code{\link{bm_taylor}()},
\code{\link{bru_get_mapper}()},
\code{\link{bru_mapper}()}
}
\concept{mappers}
