% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{bm_mesh_B}
\alias{bm_mesh_B}
\alias{bru_mapper_mesh_B}
\title{Deprecated basis conversion mapper}
\usage{
bm_mesh_B(mesh, B)

bru_mapper_mesh_B(...)
}
\arguments{
\item{mesh}{Object supported by a \code{\link[=bru_mapper]{bru_mapper()}} method.}

\item{B}{a square or tall basis conversion matrix}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
Old deprecated name for \code{\link[=bm_reparam]{bm_reparam()}}
}
\section{Functions}{
\itemize{
\item \code{bru_mapper_mesh_B()}: \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
Deprecated name for \code{bm_mesh_B}

}}
\keyword{internal}
