% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.mexdolphin.R
\name{mexdolphin}
\alias{mexdolphin}
\title{Deprecated alias for sp version of the mexdolphin dataset}
\description{
Deprecated dataset name for the \code{sp} version
of \link{mexdolphin_sf}. Use \code{\link[=mexdolphin_sp]{mexdolphin_sp()}} instead.
}
\seealso{
\link{mexdolphin_sf}
}
\keyword{internal}
