% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru_input.R, R/effect.R
\name{print.summary_bru_input}
\alias{print.summary_bru_input}
\alias{summary.bru_comp}
\alias{summary.bru_comp_list}
\alias{print.bru_comp}
\alias{print.bru_comp_list}
\alias{format.bru_subcomp}
\alias{summary.bru_subcomp}
\alias{print.bru_subcomp}
\alias{print.summary_component}
\alias{print.summary_bru_comp_list}
\alias{print.summary_bru_subcomp}
\title{Summarise components}
\usage{
\method{print}{summary_bru_input}(x, ...)

\method{summary}{bru_comp}(object, ..., depth = Inf, verbose = TRUE)

\method{summary}{bru_comp_list}(object, verbose = TRUE, ...)

\method{print}{bru_comp}(x, ...)

\method{print}{bru_comp_list}(x, ...)

\method{format}{bru_subcomp}(x, verbose = TRUE, ..., label.override = NULL)

\method{summary}{bru_subcomp}(object, verbose = TRUE, ..., label.override = NULL)

\method{print}{bru_subcomp}(x, verbose = TRUE, ..., label.override = NULL)

\method{print}{summary_component}(x, ...)

\method{print}{summary_bru_comp_list}(x, ...)

\method{print}{summary_bru_subcomp}(x, ...)
}
\arguments{
\item{x}{A summary object to be printed.}

\item{\dots}{Passed on to other summary methods.}

\item{object}{Object to be summarised.}

\item{depth}{The depth of which to expand the component mapper.
Default \code{Inf}, to traverse the entire mapper tree.}

\item{verbose}{logical; If \code{TRUE}, includes more details of the
component definitions. When \code{FALSE}, only show basic component
definition information.  Default \code{TRUE}.}

\item{label.override}{character; If not \code{NULL}, use this label instead of
the object's label.}
}
\description{
Summarise components
}
\seealso{
\code{\link[=bru_comp]{bru_comp()}}, \code{\link[=summary.bru_input]{summary.bru_input()}}
}
\author{
Fabian E. Bachl \email{bachlfab@gmail.com}

Finn Lindgren \email{finn.lindgren@gmail.com}
}
\keyword{internal}
