% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru_input.R
\name{ibm_input}
\alias{ibm_input}
\alias{ibm_input_set}
\alias{ibm_input_new}
\alias{ibm_input_available}
\alias{ibm_input_get}
\alias{bm_autodetect}
\title{Interface between \code{bru_input} and \code{bru_mapper}}
\usage{
ibm_input_set(mapper, input)

ibm_input_new(mapper, ...)

ibm_input_available(mapper)

ibm_input_get(mapper)

bm_autodetect()
}
\arguments{
\item{mapper}{A \code{bru_mapper} object.}

\item{input}{A \code{bru_input} object, or \code{NULL} to remove any existing input.
Alternatively, an existing \code{bru_mapper} object with or without associated
input can be provided, in which case the input from that mapper
is copied.}

\item{\dots}{Passed on to \code{\link[=new_bru_input]{new_bru_input()}}.}
}
\description{
Associate \link{bru_input} objects with \link{bru_mapper} objects.
}
\section{Functions}{
\itemize{
\item \code{ibm_input_set()}: Add an existing \code{bru_input} to a \code{bru_mapper}.

\item \code{ibm_input_new()}: Create and add a \code{bru_input} to a \code{bru_mapper}.

\item \code{ibm_input_available()}: Check if a \code{bru_input} is associated with a
\code{bru_mapper}.

\item \code{ibm_input_get()}: Get the \code{bru_input} associated with a \code{bru_mapper}.

\item \code{bm_autodetect()}: Create a \code{bru_mapper} placeholder object of class
\code{bm_autodetect}. The main purpose of this class is to attach \link{bru_input}
information to it, which is later used to determine a suitable 'real'
mapper type.

}}
\examples{
(m <- bm_autodetect())
ibm_input_available(m)
(m <- ibm_input_new(m, cos(x)))
ibm_input_available(m)
ibm_input_set(bm_linear(), m)

}
\seealso{
\code{\link[=bru_input]{bru_input()}}
}
