% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{bm_matrix}
\alias{bm_matrix}
\alias{bru_mapper_matrix}
\title{Mapper for matrix multiplication}
\usage{
bm_matrix(labels)

bru_mapper_matrix(...)
}
\arguments{
\item{labels}{Column labels for matrix mappings; Can be factor, character, or
a single integer specifying the number of columns for integer column
indexing.}

\item{\dots}{Arguments passed on to \code{\link[=bm_matrix]{bm_matrix()}}}
}
\description{
Create a matrix mapper, for a given number of columns
}
\examples{
m <- bm_matrix(labels = c("a", "b"))
ibm_values(m)
ibm_eval2(m, input = matrix(1:6, 3, 2), state = 2:3)

m <- bm_matrix(labels = 2L)
ibm_values(m)
ibm_eval2(m, input = matrix(1:6, 3, 2), state = 2:3)

}
\seealso{
\link{bru_mapper}, \link{bru_mapper_generics}

Other mappers: 
\code{\link{bm_aggregate}()},
\code{\link{bm_collect}()},
\code{\link{bm_const}()},
\code{\link{bm_factor}()},
\code{\link{bm_fm_mesh_1d}},
\code{\link{bm_fmesher}()},
\code{\link{bm_harmonics}()},
\code{\link{bm_index}()},
\code{\link{bm_linear}()},
\code{\link{bm_logitaverage}()},
\code{\link{bm_logsumexp}()},
\code{\link{bm_marginal}()},
\code{\link{bm_multi}()},
\code{\link{bm_pipe}()},
\code{\link{bm_reparam}()},
\code{\link{bm_repeat}()},
\code{\link{bm_scale}()},
\code{\link{bm_shift}()},
\code{\link{bm_sum}()},
\code{\link{bm_taylor}()},
\code{\link{bru_get_mapper}()},
\code{\link{bru_mapper}()}
}
\concept{mappers}
