% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/track_plotting.R
\name{bru_timings_plot}
\alias{bru_timings_plot}
\title{Plot inlabru iteration timings}
\usage{
bru_timings_plot(x)
}
\arguments{
\item{x}{a \link{bru} object, typically a result from \code{\link[=bru]{bru()}} for a nonlinear
predictor model}
}
\description{
Draws the time per iteration for preprocessing (including linearisation),
\code{inla()} calls, and
line search. Iteration \code{0} is the time used for defining the model structure.
}
\details{
Requires the "ggplot2" package to be installed.
}
\examples{
\dontrun{
fit <- bru(...)
bru_timings_plot(fit)
}
}
