% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R, R/bru_input.R, R/ggplot.R,
%   R/utils.R
\name{inlabru-deprecated}
\alias{inlabru-deprecated}
\alias{input_eval}
\alias{evaluate_inputs}
\alias{gmap}
\alias{gm}
\alias{row_kron}
\title{Deprecated functions in inlabru}
\usage{
input_eval(...)

evaluate_inputs(...)

gmap(...)

gm(...)

row_kron(M1, M2, repl = NULL, n.repl = NULL, weights = NULL)
}
\arguments{
\item{\dots}{Usually passed on to other methods}

\item{M1}{A matrix that can be transformed into a sparse Matrix.}

\item{M2}{A matrix that can be transformed into a sparse Matrix.}

\item{repl}{An optional index vector.  For each entry, specifies which
replicate the row belongs to, in the sense used in
\code{INLA::inla.spde.make.A}}

\item{n.repl}{The maximum replicate index, in the sense used in
\code{INLA::inla.spde.make.A()}.}

\item{weights}{Optional scaling weights to be applied row-wise to the
resulting matrix.}
}
\description{
Some of these functions may still attempt to do their job, but will be
removed in a future version.
}
\section{Functions}{
\itemize{
\item \code{input_eval()}: \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} from
\verb{2.12.0.9023}.
Use \code{bru_input()} instead.

\item \code{evaluate_inputs()}: \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} since
version \verb{2.12.0.9023}. Use \code{\link[=bru_input]{bru_input()}} instead. Computes the component
inputs for included components for each observation model.

\item \code{gmap()}: Plot a map using extent of a spatial object

\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} This function is deprecated as \code{ggmap}
isn't supported.

Used \code{ggmap::get_map()} to query map services like Google Maps for a region
centered around the spatial object provided. Then calls \code{ggmap()} to plot the
map.

This function required the \code{ggmap} package.

\item \code{gm()}: \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} This function is deprecated as \code{ggmap}
isn't supported.

ggplot geom for spatial data

gm is a wrapper for the \link{gg} method. It will take the first argument and
transform its coordinate system to latitude and longitude. Thereafter, \link{gg}
is called using the transformed data and the arguments provided via \code{...}. gm
is intended to replace gg whenever the data is supposed to be plotted over a
spatial map generated by \link{gmap}, which only works if the coordinate system is
latitude/longitude.

\item \code{row_kron()}: Row-wise Kronecker products
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} in favour of \code{\link[fmesher:fm_row_kron]{fmesher::fm_row_kron()}}.

Takes two Matrices and computes the row-wise Kronecker product.  Optionally
applies row-wise weights and/or applies an additional 0/1 row-wise Kronecker
matrix product.

Returns a \code{Matrix::sparseMatrix} object.

}}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}
\keyword{internal}
