% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.gorillas.R
\name{gorillas}
\alias{gorillas}
\title{Deprecated alias for sp version of the gorillas dataset}
\description{
Deprecated dataset name for the \code{sp} version
of \link{gorillas_sf}. Use \code{\link[=gorillas_sp]{gorillas_sp()}} instead.
}
\seealso{
\link{gorillas_sf}
}
\keyword{internal}
