% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inla.R
\name{bru_names}
\alias{bru_names}
\alias{bru_names.inla}
\alias{bru_names.bru}
\title{Extract standardised names from a bru or inla result object}
\usage{
bru_names(x)

\method{bru_names}{inla}(x)

\method{bru_names}{bru}(x)
}
\arguments{
\item{x}{an \code{inla} or \code{\link[=bru]{bru()}} result object}
}
\value{
A character vector with standardised names
}
\description{
Extracts the names of fixed effects, random effects, and hyperparameters,
converted with \code{\link[=bru_standardise_names]{bru_standardise_names()}}
}
\examples{
if (bru_safe_inla()) {
  fit <- bru(y ~ 1 + x + z(z, model = "iid"),
    data = data.frame(
      y = rnorm(10),
      x = rnorm(10),
      z = rep(seq_len(2), 5)
    )
  )
  bru_names(fit)
}
}
