% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/environment.R
\name{bru_call_options}
\alias{bru_call_options}
\title{Additional bru options}
\usage{
bru_call_options(...)
}
\arguments{
\item{\dots}{Options passed on to \code{\link[=as.bru_options]{as.bru_options()}}}
}
\value{
A \code{bru_options} object
}
\description{
Construct a \code{bru_options} object including the default and global options,
and converting deprecated option names.
}
\examples{
\donttest{

opts <- bru_call_options()

# Print them:
opts
}

}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}
\keyword{internal}
