% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stack.R
\name{bru_make_stack}
\alias{bru_make_stack}
\alias{bru_make_stack.bru_obs}
\alias{bru_make_stack.bru_obs_list}
\title{Build an inla data stack from linearisation information}
\usage{
bru_make_stack(...)

\method{bru_make_stack}{bru_obs}(lhood, lin, idx, ..., family_index = 1L)

\method{bru_make_stack}{bru_obs_list}(lhoods, lin, idx, ...)
}
\arguments{
\item{\dots}{Arguments passed on to other methods}

\item{lhood}{A \code{bru_obs} object}

\item{lin}{Linearisation information
\itemize{
\item For \code{.bru_obs}, a \code{bm_taylor} object
\item For \code{.bru_obs_list}, a list of \code{bm_taylor} objects
}}

\item{idx}{Output from \code{\link[=bru_index.bru_model]{bru_index.bru_model()}}}

\item{family_index}{integer specifying the family sequence index of the
observation model}

\item{lhoods}{A \code{bru_obs_list} object}
}
\description{
Combine linearisation for multiple likelihoods
}
\keyword{internal}
