% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot.R
\name{gg}
\alias{gg}
\title{ggplot2 geomes for inlabru related objects}
\usage{
gg(data, ...)
}
\arguments{
\item{data}{an object for which to generate a geom.}

\item{\dots}{Arguments passed on to the geom method.}
}
\value{
The form of the value returned by gg depends on the class of its
argument. See the documentation of the particular methods for details of
what is produced by that method.
}
\description{
gg is a generic function for generating geomes from various kinds of spatial
objects, e.g. Spatial* data, meshes, Raster objects and inla/inlabru
predictions. The function invokes particular methods which depend on the
\link{class} of the first argument.
}
\examples{
if (require("ggplot2", quietly = TRUE)) {
  # Load Gorilla data

  gorillas <- inlabru::gorillas_sf

  # Invoke ggplot and add geomes for the Gorilla nests and the survey
  # boundary

  ggplot() +
    gg(gorillas$boundary) +
    gg(gorillas$nests)
}
}
\seealso{
Other geomes: 
\code{\link{gg.RasterLayer}()},
\code{\link{gg.SpatRaster}()},
\code{\link{gg.Spatial}},
\code{\link{gg.data.frame}()},
\code{\link{gg.fm_mesh_1d}()},
\code{\link{gg.fm_mesh_2d}()},
\code{\link{gg.matrix}()},
\code{\link{gg.sf}()}
}
\concept{geomes}
