% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inla.R
\name{bru_inla.stack.mexpand}
\alias{bru_inla.stack.mexpand}
\title{Backwards compatibility to handle mexpand for INLA <= 24.06.02}
\usage{
bru_inla.stack.mexpand(
  ...,
  old.names = "BRU.response",
  new.name = "BRU.response"
)
}
\arguments{
\item{\dots}{List of stacks that contain vector observations (existing
multilikelihood observation matrices are also permitted)}

\item{old.names}{A vector of strings with the names of the observation
vector/matrix for each stack. If a single string, this is assumed for all
the stacks. (default "BRU.response")}

\item{new.name}{The name to be used for the expanded observation matrix,
possibly the same as an old name. (default "BRU.response")}
}
\value{
a list of modified stacks with multicolumn observations
}
\description{
Expand observation vectors/matrices in stacks into to a multicolumn matrix
for multiple likelihoods
}
\author{
Fabian E. Bachl \email{f.e.bachl@bath.ac.uk} and Finn Lindgren
\email{finn.lindgren@gmail.com}
}
\keyword{internal}
