% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru.inference.R
\name{bru_info}
\alias{bru_info}
\alias{bru_info.character}
\alias{bru_info.bru}
\alias{as_bru_info}
\alias{as_bru_info.bru_info}
\alias{as_bru_info.bru}
\alias{summary.bru_info}
\alias{print.summary_bru_info}
\alias{print.bru_info}
\title{Methods for bru_info objects}
\usage{
bru_info(...)

\method{bru_info}{character}(method, ..., inlabru_version = NULL, INLA_version = NULL)

\method{bru_info}{bru}(object, ...)

as_bru_info(object, ...)

\method{as_bru_info}{bru_info}(object, ...)

\method{as_bru_info}{bru}(object, ...)

\method{summary}{bru_info}(object, verbose = TRUE, ...)

\method{print}{summary_bru_info}(x, ...)

\method{print}{bru_info}(x, ...)
}
\arguments{
\item{\dots}{Additional arguments to be stored in the \code{bru_info} object. For
\code{summary} and \code{print} methods, arguments passed on to submethods.}

\item{method}{character; The type of estimation method used}

\item{inlabru_version}{character; inlabru package version. Default: NULL, for
automatically detecting the version}

\item{INLA_version}{character; INLA package version. Default: NULL, for
automatically detecting the version}

\item{object}{A \code{bru_info} object}

\item{verbose}{logical; If \code{TRUE}, include more details of the
component definitions. If \code{FALSE}, only show basic component
definition information. Default: \code{FALSE}}

\item{x}{An  object to be printed}
}
\description{
The \code{bru_info} class is used to store metadata about \code{bru} models.
}
\section{Methods (by class)}{
\itemize{
\item \code{bru_info(character)}: Create a \code{bru_info} object

\item \code{bru_info(bru)}: Extract the \code{bru_info} object from an estimated \code{\link[=bru]{bru()}}
result object. The default print method shows information about model
components and observation models.

}}
\section{Methods (by generic)}{
\itemize{
\item \code{as_bru_info(bru_info)}: Extract a \code{bru_info} object.

}}
\section{Functions}{
\itemize{
\item \code{as_bru_info()}: Extract the \code{bru_info} object from an estimated \code{\link[=bru]{bru()}}
result object. The default print method shows information about model
components and observation models.

\item \code{as_bru_info(bru)}: Extract the \code{bru_info} object from an estimated \code{\link[=bru]{bru()}}
result object.

}}
