% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{ibm_is_rowwise}
\alias{ibm_is_rowwise}
\alias{ibm_is_rowwise.default}
\title{Check if a mapper is rowwise}
\usage{
ibm_is_rowwise(mapper, ...)

\method{ibm_is_rowwise}{default}(mapper, ...)
}
\arguments{
\item{mapper}{A mapper S3 object, inheriting from \code{bru_mapper}.}

\item{...}{Arguments passed on to other methods}
}
\description{
Implementations must return \code{TRUE} or \code{FALSE}.
If \code{TRUE} (returned by the default method unless the mapper
contains an \code{is_rowwise} variable), users of the mapper
may assume the mapper uses its inputs in "rowwise" manner, so that
blockwise evaluation is always possible.
}
\section{Methods (by class)}{
\itemize{
\item \code{ibm_is_rowwise(default)}: Returns logical
\code{is_rowwise} from the mapper object if it exists, and otherwise \code{TRUE}.

}}
\seealso{
Other mapper methods: 
\code{\link{bru_mapper_generics}},
\code{\link{ibm_eval}()},
\code{\link{ibm_eval2}()},
\code{\link{ibm_inla_subset}()},
\code{\link{ibm_invalid_output}()},
\code{\link{ibm_is_linear}()},
\code{\link{ibm_jacobian}()},
\code{\link{ibm_linear}()},
\code{\link{ibm_n}()},
\code{\link{ibm_n_output}()},
\code{\link{ibm_names}()},
\code{\link{ibm_simplify}()},
\code{\link{ibm_values}()}
}
\concept{mapper methods}
