% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru_is.R
\name{bru_is_rowwise}
\alias{bru_is_rowwise}
\alias{bru_is_rowwise.bru_pred_expr}
\alias{bru_is_rowwise.bru_obs}
\alias{bru_is_rowwise.bru_obs_list}
\alias{bru_is_rowwise.bru_comp_list}
\alias{bru_is_rowwise.bru_comp}
\alias{bru_is_rowwise.bru_mapper}
\title{Check for predictor rowwise evaluability}
\usage{
bru_is_rowwise(x, ...)

\method{bru_is_rowwise}{bru_pred_expr}(x, ...)

\method{bru_is_rowwise}{bru_obs}(x, ...)

\method{bru_is_rowwise}{bru_obs_list}(x, ...)

\method{bru_is_rowwise}{bru_comp_list}(x, ...)

\method{bru_is_rowwise}{bru_comp}(x, ...)

\method{bru_is_rowwise}{bru_mapper}(x, ...)
}
\arguments{
\item{x}{An object containing a predictor definition}

\item{\dots}{Arguments passed on to submethods.}
}
\value{
\code{TRUE} if the expression is believed to be rowwise, \code{FALSE}
otherwise.
}
\description{
Checks if a predictor expression may be evaluated rowwise
}
\examples{
bru_is_rowwise(new_bru_pred_expr(~ x + y))

}
