% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R
\name{bm_pipe}
\alias{bm_pipe}
\alias{bru_mapper_pipe}
\title{Mapper for linking several mappers in sequence}
\usage{
bm_pipe(mappers)

bru_mapper_pipe(...)
}
\arguments{
\item{mappers}{A list of \code{bru_mapper} objects}

\item{\dots}{Arguments passed on to \code{\link[=bm_pipe]{bm_pipe()}}}
}
\description{
Create a pipe mapper, where \code{mappers} is a list of mappers,
and the evaluated output of each mapper is handed as the state to the next
mapper.
The \code{input} format for the \code{ibm_eval} and \code{ibm_jacobian} methods is
a list of inputs, one for each mapper.
}
\examples{
m <- bm_pipe(list(
  scale = bm_scale(),
  shift = bm_shift()
))
ibm_eval2(m, input = list(scale = 2, shift = 1:4), state = 1:4)

}
\seealso{
\link{bru_mapper}, \link{bru_mapper_generics}

Other mappers: 
\code{\link{bm_aggregate}()},
\code{\link{bm_collect}()},
\code{\link{bm_const}()},
\code{\link{bm_factor}()},
\code{\link{bm_fm_mesh_1d}},
\code{\link{bm_fmesher}()},
\code{\link{bm_harmonics}()},
\code{\link{bm_index}()},
\code{\link{bm_linear}()},
\code{\link{bm_logitaverage}()},
\code{\link{bm_logsumexp}()},
\code{\link{bm_marginal}()},
\code{\link{bm_matrix}()},
\code{\link{bm_multi}()},
\code{\link{bm_reparam}()},
\code{\link{bm_repeat}()},
\code{\link{bm_scale}()},
\code{\link{bm_shift}()},
\code{\link{bm_sum}()},
\code{\link{bm_taylor}()},
\code{\link{bru_get_mapper}()},
\code{\link{bru_mapper}()}
}
\concept{mappers}
