% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/environment.R
\name{bru_env_get}
\alias{bru_env_get}
\title{Get access to the internal environment}
\usage{
bru_env_get()
}
\description{
Get access to the internal environment
}
\details{
The environment is defined in \verb{0_inlabru_envir.R} which is loaded
first.
}
\keyword{internal}
