% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bru.inference.R
\name{bru_timings}
\alias{bru_timings}
\alias{bru_timings.bru}
\title{Extract timing information from fitted \link{bru} object}
\usage{
bru_timings(object, ...)

\method{bru_timings}{bru}(object, ...)
}
\arguments{
\item{object}{A fitted \code{bru} object}

\item{\dots}{unused}
}
\description{
Extracts a data.frame or tibble with information about the \code{Time} (CPU),
\code{System}, and \code{Elapsed} time for each step of a \code{bru()} run.
}
