% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mappers.R, R/mapper_collect.R, R/mapper_sum.R
\name{ibm_names}
\alias{ibm_names}
\alias{ibm_names<-}
\alias{ibm_names.default}
\alias{ibm_names.bm_multi}
\alias{ibm_names<-.bm_multi}
\alias{ibm_names<-.bru_mapper_multi}
\alias{ibm_names.bm_collect}
\alias{ibm_names<-.bm_collect}
\alias{ibm_names<-.bru_mapper_collect}
\alias{ibm_names.bm_sum}
\alias{ibm_names<-.bm_sum}
\alias{ibm_names<-.bru_mapper_sum}
\title{Names of submapper}
\usage{
ibm_names(mapper)

ibm_names(mapper) <- value

\method{ibm_names}{default}(mapper, ...)

\method{ibm_names}{bm_multi}(mapper)

\method{ibm_names}{bm_multi}(mapper) <- value

\method{ibm_names}{bru_mapper_multi}(mapper) <- value

\method{ibm_names}{bm_collect}(mapper)

\method{ibm_names}{bm_collect}(mapper) <- value

\method{ibm_names}{bru_mapper_collect}(mapper) <- value

\method{ibm_names}{bm_sum}(mapper)

\method{ibm_names}{bm_sum}(mapper) <- value

\method{ibm_names}{bru_mapper_sum}(mapper) <- value
}
\arguments{
\item{mapper}{A mapper S3 object, inheriting from \code{bru_mapper}.}

\item{value}{a character vector of up to the same length as the number
of mappers in the multi-mapper x}

\item{...}{Arguments passed on to other methods}
}
\description{
Implementations must return a character vector of sub-mapper names, or
\code{NULL}. Intended for providing information about multi-mappers and mapper
collections.
}
\section{Methods (by class)}{
\itemize{
\item \code{ibm_names(default)}: Returns \code{NULL}

\item \code{ibm_names(bm_multi)}: Returns the names from the sub-mappers list

\item \code{ibm_names(bm_collect)}: Returns the names from the
sub-mappers list

\item \code{ibm_names(bm_sum)}: Returns the names from the sub-mappers list

}}
\section{Functions}{
\itemize{
\item \code{ibm_names(mapper) <- value}: Set mapper names.

}}
\examples{
# ibm_names
mapper <- bm_multi(list(A = bm_index(2), B = bm_index(2)))
ibm_names(mapper)
ibm_names(mapper) <- c("new", "names")
ibm_names(mapper)
}
\seealso{
Other mapper methods: 
\code{\link{bru_mapper_generics}},
\code{\link{ibm_eval}()},
\code{\link{ibm_eval2}()},
\code{\link{ibm_inla_subset}()},
\code{\link{ibm_invalid_output}()},
\code{\link{ibm_is_linear}()},
\code{\link{ibm_is_rowwise}()},
\code{\link{ibm_jacobian}()},
\code{\link{ibm_linear}()},
\code{\link{ibm_n}()},
\code{\link{ibm_n_output}()},
\code{\link{ibm_simplify}()},
\code{\link{ibm_values}()}

Other mapper methods: 
\code{\link{bru_mapper_generics}},
\code{\link{ibm_eval}()},
\code{\link{ibm_eval2}()},
\code{\link{ibm_inla_subset}()},
\code{\link{ibm_invalid_output}()},
\code{\link{ibm_is_linear}()},
\code{\link{ibm_is_rowwise}()},
\code{\link{ibm_jacobian}()},
\code{\link{ibm_linear}()},
\code{\link{ibm_n}()},
\code{\link{ibm_n_output}()},
\code{\link{ibm_simplify}()},
\code{\link{ibm_values}()}
}
\concept{mapper methods}
