% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hexagon_tiling.R
\name{cv_partition}
\alias{cv_partition}
\title{Grid partitioning}
\usage{
cv_partition(
  samplers,
  resolution = NULL,
  nrows = NULL,
  ncols = NULL,
  chess = TRUE,
  ...
)
}
\arguments{
\item{samplers}{A sf object containing region for which
partitions to be created}

\item{resolution}{resolution of the grids that are required}

\item{nrows}{number of rows of grids that are required}

\item{ncols}{number of columns of grids that are required}

\item{chess}{chessboard partitioning}

\item{\dots}{Unused.}
}
\value{
a partitioned sf object as required or a list of partitioned sf
objects if chess is TRUE
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}} Both the interface and function name
may change in future versions.

Partitions the region based on the given criteria for calculating residuals
in each partition. Parts of this function are taken from concepts in
https://rpubs.com/huanfaChen/grid_from_polygon
}
\author{
Man Ho Suen
}
\keyword{internal}
